@@ -38,6 +38,7 @@ import java.sql.Statement;
 import java.sql.Timestamp;
 import java.sql.Types;
 import java.util.ArrayList;
+import java.util.Base64;
 import java.util.Collections;
 import java.util.Date;
 import java.util.HashMap;
@@ -81,7 +82,6 @@ import org.jboss.msc.service.StartContext;
 import org.jboss.msc.service.StartException;
 import org.jboss.msc.service.StopContext;
 import org.jboss.msc.value.InjectedValue;
-import org.jboss.util.Base64;
 
 /**
  * <p>
@@ -400,7 +400,7 @@ public class DatabaseTimerPersistence implements TimerPersistence, Service<Datab
                 if (timer.getNextExpiration() == null) {
                     statement.setTimestamp(6, null);
                 } else {
-                    statement.setTimestamp(6, new Timestamp(timer.getNextExpiration().getTime()));
+                    statement.setTimestamp(6, timestamp(timer.getNextExpiration()));
                 }
             } catch (SQLException e) {
                 // something wrong with the preparation
@@ -452,10 +452,10 @@ public class DatabaseTimerPersistence implements TimerPersistence, Service<Datab
             statement.setString(1, timedObjectId);
             statement.setString(2, partition);
             resultSet = statement.executeQuery();
-            final List<TimerImpl> timers = new ArrayList<TimerImpl>();
+            final List<Holder> timers = new ArrayList<>();
             while (resultSet.next()) {
                 try {
-                    final TimerImpl timerImpl = timerFromResult(resultSet, timerService);
+                    final Holder timerImpl = timerFromResult(resultSet, timerService);
                     if (timerImpl != null) {
                         timers.add(timerImpl);
                     }
@@ -465,13 +465,30 @@ public class DatabaseTimerPersistence implements TimerPersistence, Service<Datab
             }
             synchronized (this) {
                 Set<String> ids = new HashSet<>();
-                for (TimerImpl timer : timers) {
-                    ids.add(timer.getId());
+                for (Holder timer : timers) {
+                    ids.add(timer.timer.getId());
                 }
                 knownTimerIds.put(timedObjectId, ids);
+                for(Holder timer : timers) {
+                    if(timer.requiresReset) {
+                        TimerImpl ret = timer.timer;
+                        EjbLogger.DEPLOYMENT_LOGGER.loadedPersistentTimerInTimeout(ret.getId(), ret.getTimedObjectId());
+                        if(ret.getNextExpiration() == null) {
+                            ret.setTimerState(TimerState.CANCELED);
+                            persistTimer(ret);
+                            return null;
+                        } else {
+                            ret.setTimerState(TimerState.ACTIVE);
+                            persistTimer(ret);
+                        }
+                    }
+                }
             }
-
-            return timers;
+            List<TimerImpl> ret = new ArrayList<>();
+            for(Holder timer : timers) {
+                ret.add(timer.timer);
+            }
+            return ret;
         } catch (SQLException e) {
             throw new RuntimeException(e);
         } finally {
@@ -497,8 +514,10 @@ public class DatabaseTimerPersistence implements TimerPersistence, Service<Datab
         return this;
     }
 
-    private TimerImpl timerFromResult(final ResultSet resultSet, final TimerServiceImpl timerService) throws SQLException {
+    private Holder timerFromResult(final ResultSet resultSet, final TimerServiceImpl timerService) throws SQLException {
         boolean calendarTimer = resultSet.getBoolean(24);
+        final String nodeName = resultSet.getString(25);
+        boolean requiresReset = false;
 
         TimerImpl.Builder builder = null;
         if (calendarTimer) {
@@ -544,7 +563,14 @@ public class DatabaseTimerPersistence implements TimerPersistence, Service<Datab
         builder.setInfo((Serializable) deSerialize(resultSet.getString(8)));
         builder.setTimerState(TimerState.valueOf(resultSet.getString(9)));
         builder.setPersistent(true);
-        return builder.build(timerService);
+
+        TimerImpl ret =  builder.build(timerService);
+        if(nodeName != null && (nodeName.equals(this.nodeName))) {
+            if(ret.getState() == TimerState.IN_TIMEOUT || ret.getState() == TimerState.RETRY_TIMEOUT) {
+                requiresReset = true;
+            }
+        }
+        return new Holder(ret, requiresReset);
     }
 
     private void statementParameters(final TimerImpl timerEntity, final PreparedStatement statement) throws SQLException {
@@ -624,14 +650,14 @@ public class DatabaseTimerPersistence implements TimerPersistence, Service<Datab
         } catch (IOException e) {
             throw new RuntimeException(e);
         }
-        return Base64.encodeBytes(out.toByteArray());
+        return Base64.getEncoder().encodeToString(out.toByteArray());
     }
 
     public Object deSerialize(final String data) throws SQLException {
         if (data == null) {
             return null;
         }
-        InputStream in = new ByteArrayInputStream(Base64.decode(data));
+        InputStream in = new ByteArrayInputStream(Base64.getDecoder().decode(data));
         try {
             final Unmarshaller unmarshaller = factory.createUnmarshaller(configuration);
             unmarshaller.start(new InputStreamByteInput(in));
@@ -651,7 +677,14 @@ public class DatabaseTimerPersistence implements TimerPersistence, Service<Datab
         if (date == null) {
             return null;
         }
-        return new Timestamp(date.getTime());
+        long time = date.getTime();
+        if(database != null && database.equals("mysql")) {
+            // truncate the milliseconds because MySQL 5.6.4+ and MariaDB 5.3+ do the same
+            // and querying with a Timestamp containing milliseconds doesn't match the rows
+            // with such truncated DATETIMEs
+            time -= time % 1000;
+        }
+        return new Timestamp(time);
     }
 
     /**
@@ -756,8 +789,10 @@ public class DatabaseTimerPersistence implements TimerPersistence, Service<Datab
                                         synchronized (DatabaseTimerPersistence.this) {
                                             knownTimerIds.get(timedObjectId).add(id);
                                         }
-                                        final TimerImpl timerImpl = timerFromResult(resultSet, listener.getTimerService());
-                                        listener.timerAdded(timerImpl);
+                                        final Holder holder = timerFromResult(resultSet, listener.getTimerService());
+                                        if(holder != null) {
+                                            listener.timerAdded(holder.timer);
+                                        }
                                     }
                                 } catch (Exception e) {
                                     EjbLogger.ROOT_LOGGER.timerReinstatementFailed(resultSet.getString(2), resultSet.getString(1), e);
@@ -786,4 +821,23 @@ public class DatabaseTimerPersistence implements TimerPersistence, Service<Datab
 
         }
     }
+
+
+    static final class Holder {
+        final TimerImpl timer;
+        final boolean requiresReset;
+
+        Holder(TimerImpl timer, boolean requiresReset) {
+            this.timer = timer;
+            this.requiresReset = requiresReset;
+        }
+
+        public TimerImpl getTimer() {
+            return timer;
+        }
+
+        public boolean isRequiresReset() {
+            return requiresReset;
+        }
+    }
 }
